/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.manager;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.config.*;
import de.bsvrz.pua.prot.client.dataobject.ProtocolResult;
import de.bsvrz.pua.prot.sender.PuaSender;
import de.bsvrz.pua.prot.util.JobInProgress;
import de.bsvrz.pua.prot.util.ProcessingParameter;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.datk.AtgProtocolRequest;
import de.bsvrz.sys.funclib.losb.datk.PidProtocol;
import de.bsvrz.sys.funclib.losb.datk.SerializerUtil;
import de.bsvrz.sys.funclib.losb.exceptions.FailureException;

import java.util.ArrayList;

/**
 * Klasse zum Versenden einer Antwort auf eine Protokoll-bezogene Anfrage. Der Sendevorgang wird als einfacher Sender durchgeführt. Verwendet {@link
 * de.bsvrz.sys.funclib.losb.kernsoftware.Sender} zum Versenden der Daten.
 *
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $Revision$ / $Date$ / ($Author$)
 */
@SuppressWarnings("deprecation")
public class ProtocolAnswer {

	private final Debug _debug = Debug.getLogger();

	/**
	 * Sendet den Status an den Auftraggeber. Wartet, bis die Daten gesendet wurden oder es zu einem Sendefehler kommt. Meldet sich vor dem Senden als einfacher
	 * sender an. Meldet sich nach dem Senden sofort wieder ab. Der Sendevorgang wird als einfacher Sender durchgeführt.
	 *
	 * @param protocolId Protokoll-Id
	 * @param status     Status (in Prozent)
	 * @param sender Sender
	 * @throws FailureException Falls nicht gesendet werden konnte.
	 * @return                <code>true</code> falls gesendet wurde.<code>false</code> bei negativer Sendesteuerung durch den Datenverteiler (weil kein Abnehmer).
	 */
	public static boolean sendStatus(PuaSender sender, long protocolId, byte status) throws FailureException {
		Data data = AtgProtocolRequest.build(sender.getDav(), sender.getObject(), protocolId, ProtocolRelatedOperation.STATUS, new byte[]{status});
		sender.send(data);
		return sender.isAlive();
	}

	/**
	 * Sendet die Liste mit den aktuellen Aufträgen an den Auftraggeber. Wartet, bis die Daten gesendet wurden oder es zu einem Sendefehler kommt. Meldet sich
	 * vor dem Senden als einfacher sender an. Meldet sich nach dem Senden sofort wieder ab. Der Sendevorgang wird als einfacher Sender durchgeführt.
	 *
	 * @param protocolId Protokoll-Id
	 * @param jobs       Auftragsliste
	 * @param sender Sender
	 * @throws FailureException Falls nicht gesendet werden konnte.
	 * @return                <code>true</code> falls gesendet wurde.<code>false</code> bei negativer Sendesteuerung durch den Datenverteiler (weil kein Abnehmer).
	 */
	public static boolean sendJobList(PuaSender sender, long protocolId, ArrayList<JobInProgress> jobs) throws FailureException {
		Data data = AtgProtocolRequest.build(sender.getDav(), sender.getObject(), protocolId, ProtocolRelatedOperation.ACK_FLAG, SerializerUtil.serializeToByteArray(jobs));
		sender.send(data);
		return sender.isAlive();
	}

	/**
	 * Sendet den Status von PuA bzw. dem Archivsystem an den Auftraggeber. Der Sendevorgang wird als einfacher Sender durchgeführt.
	 *
	 * @param protocolId       Protokoll-Id
	 * @param archiveAvailable Status (in Prozent)
	 * @param sender Sender
	 * @throws FailureException Falls nicht gesendet werden konnte.
	 */
	public static void sendPuaArsStatus(PuaSender sender, final long protocolId, final boolean archiveAvailable)
			throws FailureException {
		final Data data = AtgProtocolRequest.build(sender.getDav(), sender.getObject(), protocolId, ProtocolRelatedOperation.PUA_ARS_STATUS, new byte[]{
				archiveAvailable ? (byte) 1 : (byte) 0
		});
		sender.send(data);
		sender.isAlive();
	}

	/**
	 * Sendet eine Fehlermeldung an den Auftraggeber. Wartet, bis die Daten gesendet wurden oder es zu einem Sendefehler kommt. Der Sendevorgang wird als
	 * einfacher Sender durchgeführt.
	 *
	 * @param sender    Objekt das den Datenversand durchführt.
	 * @param requestId Anfrage-Id
	 * @param errorMsg  Fehlermeldung
	 * @throws FailureException Falls nicht gesendet werden konnte.
	 * @return                <code>true</code> falls gesendet wurde.<code>false</code> bei negativer Sendesteuerung durch den Datenverteiler (weil kein Abnehmer).
	 */
	public static boolean sendError(PuaSender sender, long requestId, String errorMsg) throws FailureException {
		Data answer = AtgProtocolRequest.build(sender.getDav(), sender.getObject(), requestId, ProtocolRelatedOperation.NACK_FLAG, SerializerUtil.serializeToByteArray(errorMsg));
		sender.send(answer);
		return sender.isAlive();
	}

	/**
	 * Sendet eine Quittung an den Auftraggeber. Wartet, bis die Daten gesendet wurden oder es zu einem Sendefehler kommt. Der Sendevorgang wird als einfacher
	 * Sender durchgeführt.
	 *
	 * @param sender    Objekt das den Datenversand durchführt.
	 * @param requestId Anfrage-Id
	 * @throws FailureException Falls nicht gesendet werden konnte.
	 * @return                <code>true</code> falls gesendet wurde.<code>false</code> bei negativer Sendesteuerung durch den Datenverteiler (weil kein Abnehmer).
	 */
	public static boolean sendAck(PuaSender sender, long requestId) throws FailureException {
		byte[] data = SerializerUtil.serializeToByteArray(new byte[0]);
		Data answer = AtgProtocolRequest.build(sender.getDav(), sender.getObject(), requestId, ProtocolRelatedOperation.ACK_FLAG, data);
		sender.send(answer);
		return sender.isAlive();
	}

	/**
	 * Sendet eine Liste mit den gespeicherten Protokollen an den Auftraggeber. Wartet, bis die Daten gesendet wurden oder es zu einem Sendefehler kommt. Der
	 * Sendevorgang wird als einfacher Sender durchgeführt. Aufbau der gesendeten Daten:<br> Byte 0 - x: long[] mit Protokoll-Ids<br> Byte x+1 - y:
	 * ProcessinParameter[] mit ProtokollInformationen.
	 *
	 * @param requestId   Anfrage-Id
	 * @param protocolIds Liste mit den Ids der gespeicherten Protokolle
	 * @param pp          Weitere Informationen zum Protokoll.
	 * @param sender Sender
	 * @throws FailureException Falls nicht gesendet werden konnte.
	 * @return                <code>true</code> falls gesendet wurde.<code>false</code> bei negativer Sendesteuerung durch den Datenverteiler (weil kein Abnehmer).
	 */
	public static boolean sendProtocolList(PuaSender sender, long requestId, long[] protocolIds, ProcessingParameter[] pp)
			throws FailureException {
		byte[][] baPP = new byte[pp.length + 1][];
		baPP[0] = SerializerUtil.serializeToByteArray(protocolIds);
		//int length = 0;
		for(int i = 0; i < pp.length; i++) {
			baPP[i + 1] = pp[i].serialize();
			//length += baPP[i].length;
		}

		Data data = AtgProtocolRequest.build(sender.getDav(), sender.getObject(), requestId, ProtocolRelatedOperation.ACK_FLAG, SerializerUtil.serializeToByteArray(baPP));
		sender.send(data);
		return sender.isAlive();
	}


	/**
	 * Sendet den Protokolldatensatz als einfacher Sender.
	 *
	 * @param sender         Objekt das den Datenversand durchführt.
	 * @param jobId          Protokoll-Id
	 * @param serializer     Serialisierer-Objekt.
	 * @param protocolResult Ergebnisdatensatz.
	 * @throws FailureException Fehler beim Senden.
	 * @return                    <code>true</code> falls gesendet wurde.<code>false</code> bei negativer Sendesteuerung durch den Datenverteiler (weil kein Abnehmer).
	 */
	public static boolean sendProtocol(PuaSender sender, long jobId, SerializerUtil serializer, ProtocolResult protocolResult) throws FailureException {
		Data data = AtgProtocolRequest.build(sender.getDav(), sender.getObject(), jobId, ProtocolRelatedOperation.CREATE, serializer, protocolResult);
		sender.send(data);
		return sender.isAlive();
	}


	/**
	 * Sendet die Auftrags-Id als einfacher Sender an den Auftraggeber. Wartet, bis die Daten gesendet wurden oder es zu einem Sendefehler kommt.
	 * <p>
	 * Clientseitig wird die Request-ID ab dem erstem (diesem) Datensatz durch die Job-ID ersetzt unter der dann die weitere Kommunikation abgewickelt wird.
	 *
	 * @param requestId Anfrage-Id
	 * @param jobId     Auftrags-Id.
	 * @param sender Sender
	 * @throws FailureException Falls nicht gesendet werden konnte.
	 * @return                <code>true</code> falls gesendet wurde.<code>false</code> bei negativer Sendesteuerung durch den Datenverteiler (weil kein Abnehmer).
	 */
	public static boolean sendJobId(PuaSender sender, long requestId, long jobId) throws FailureException {
		Data data = AtgProtocolRequest.build(sender.getDav(), sender.getObject(), requestId, ProtocolRelatedOperation.ACK_FLAG, SerializerUtil.serializeId(jobId));
		sender.send(data);
		return sender.isAlive();
	}
	/**
	 * Sendet eine negative Quittung, die zeigt, dass die Protokollerstellung abgebrochen wurde. Im Datenteil wird der Grund des Fehlers mitgeliefert.
	 *
	 * @param sender    Sender
	 * @param requestId Protokoll-Id
	 * @param errorMsg  Fehlermeldung.
	 * @throws FailureException Fehler beim Senden.
	 * @return                <code>true</code> falls gesendet wurde.<code>false</code> bei negativer Sendesteuerung durch den Datenverteiler (weil kein Abnehmer).
	 */
	public static boolean sendAbort(PuaSender sender, long requestId, String errorMsg) throws FailureException {
		byte[] data;
		if(errorMsg != null) {
			data = SerializerUtil.serializeToByteArray(errorMsg);
		}
		else {
			data = new byte[0];
		}
		Data answer = AtgProtocolRequest.build(sender.getDav(), sender.getObject(), requestId, ProtocolRelatedOperation.ABORT, data);
		sender.send(answer);
		return sender.isAlive();
	}

	public static PuaSender sender(final ClientDavInterface dav, final SystemObject client) {
		return new PuaSender(dav, client, getDataDescription(dav.getDataModel()), () -> {});
	}

	/**
	 * Erzeugt die passende Datenbeschreibung zum Senden.
	 *
	 * @param model Datenmodel.
	 * @return Datenbeschreibung.
	 */
	private static DataDescription getDataDescription(DataModel model) {
		AttributeGroup atg = model.getAttributeGroup(PidProtocol.atgProtocolRequest);
		Aspect asp = model.getAspect(PidProtocol.aspAnswer);
		return new DataDescription(atg, asp);
	}
}
