/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.util;

import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationException;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pua.prot.exceptions.ScopeException;
import de.bsvrz.pua.prot.interpreter.InterpreterErrorMessage;
import de.bsvrz.sys.funclib.losb.exceptions.FailureException;
import de.bsvrz.sys.funclib.losb.exceptions.LoggerException;
import de.bsvrz.sys.funclib.losb.messages.ErrorMessage;

import java.util.Objects;

/**
 * Modeliert eine Qualifizierung bestehend aus Objekt(typ), Attributgruppe, Aspekt und Simulationsvariante Entspricht der Klasse
 * <code>sys.funclib.data.DataIdentification</code> der Kernsoftware, unterstützt jedoch freie Aspekte.
 *
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $Revision$ / $Date$ / ($Author$)
 */
@SuppressWarnings("deprecation")
public class Qualifier implements Cloneable {

	/** Zeigt ungültige Simulationsvariante an */
	public static final short INVALID_SIMVAR = -1;

	/** Objekt */
	public SystemObject object;

	/** Attributgruppe */
	public AttributeGroup attributeGroup;

	/** Aspekt. Muss <code>null</code> sein, falls {@link #freeAspect} verwendet wird. */
	public Aspect aspect;

	/** Ungebundener Aspekt. Muss <code>null</code> sein, falls {@link #aspect} verwendet wird. */
	public String freeAspect;

	/** Simulationsvariante */
	public short simVar;

	/**
	 * Erzeugt ein Qualifier Objekt aus den gegebenen Objekten
	 *
	 * @param obj    Objekt
	 * @param atg    Attributgruppe
	 * @param asp    Aspekt
	 * @param simVar Simulationsvariante
	 */
	public Qualifier(SystemObject obj, AttributeGroup atg, Aspect asp, short simVar) {
		object = obj;
		attributeGroup = atg;
		aspect = asp;
		freeAspect = null;
		this.simVar = simVar;
	}

	/**
	 * Erzeugt ein Qualifier Objekt aus den gegebenen Objekten
	 *
	 * @param obj     Objekt
	 * @param atg     Attributgruppe
	 * @param freeAsp Freier Aspekt (Platzhalter)
	 * @param simVar  Simulationsvariante
	 */
	public Qualifier(SystemObject obj, AttributeGroup atg, String freeAsp, short simVar) {
		this(obj, atg, (Aspect)null, simVar);
		freeAspect = freeAsp;
	}

	/** Erzeugt Qualifier Objekt dessen Membervariablen auf null und {@link #simVar} auf {@link #INVALID_SIMVAR} gesetzt wird. */
	protected Qualifier() {
		this(null, null, (Aspect)null, Qualifier.INVALID_SIMVAR);
	}

	/**
	 * Erzeugt Qualifizierung.
	 *
	 * @param simVar Simulationsvariante.
	 */
	public Qualifier(short simVar) {
		this();
		this.simVar = simVar;
	}

	/**
	 * Erzegut ein Qualifier Objekt mit aus den gegebenen Strings. Es wird nicht überprüft, ob es sich um eine gültige Qualifizierung handelt.
	 *
	 * @param model  Datenmodell
	 * @param objPID PID des Objekts / Objekttyps
	 * @param atgPID PID der Attributgruppe
	 * @param aspPID PID des Aspekts / Name des Freien Aspekts
	 * @param simVar Simulationsvariante
	 *
	 * @throws FailureException Verwenden einer ungültigen Pid oder Kommunikationsfehler mit der Konfiguration.
	 */
	public Qualifier(DataModel model, String objPID, String atgPID, String aspPID, short simVar) throws FailureException {
		try {
			if(objPID != null) {
				object = model.getObject(objPID);
			}
			else {
				object = null;
			}
			if(atgPID != null) {
				attributeGroup = model.getAttributeGroup(atgPID);
			}
			else {
				attributeGroup = null;
			}
			if(aspPID != null) {
				aspect = model.getAspect(aspPID);
				if(aspect == null) freeAspect = aspPID;
			}
			else {
				aspect = null;
			}
			this.simVar = simVar;
		}
		catch(ConfigurationException e) {
			throw new FailureException(ErrorMessage.INVALID_PID, LoggerException.WARNING);
		}
	}


	/**
	 * Liefert Aspekt zurück. Handelt es sich um einen ungebundenen Aspekt, so wird der Platzhalter zurückgegeben. Ansonsten wird die Pid des Aspekts geliefert.
	 *
	 * @return Liefert die Pid oder den Platzhalter des Apskets zurück. Ist kein Aspekt gesetzt, so wird ein leerer String zurückgegeben.
	 */
	public String getAspect() {
		if(aspect != null) {
			return aspect.getPid();
		}
		else if(freeAspect != null) {
			return freeAspect;
		}
		else {
			return "";
		}
	}


	/**
	 * Ausgabe der Elemente
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
    public String toString() {
		StringBuilder result = new StringBuilder();
		if(object != null) result.append(object.getPid());

		if(attributeGroup != null) result.append(":" + attributeGroup.getPid());

		result.append(":" + getAspect());

		if(simVar != Qualifier.INVALID_SIMVAR) result.append(":" + simVar);

		return result.toString();
	}

	/**
	 * Klont die Qualizifierung.
	 *
	 * @return Kopie der Qualifizierung.
	 *
	 * @see java.lang.Object#clone()
	 */
	@Override
    public Qualifier clone() {
		Qualifier result = new Qualifier(object, attributeGroup, aspect, simVar);
		result.freeAspect = this.freeAspect;
		return result;
	}

	/**
	 * Berechnet den Hashwert des Objekts
	 *
	 * @return Hashwert des Objekts
	 *
	 * @see java.lang.Object#hashCode()
	 */
	@Override
    public int hashCode() {
		long result = 0;

		if(object != null) result = object.getId() % Integer.MAX_VALUE;
		if(attributeGroup != null) result = 37 * result + attributeGroup.getId() % Integer.MAX_VALUE;
		if(aspect != null) {
			result = 37 * result + aspect.getId() % Integer.MAX_VALUE;
		}
		else if(freeAspect != null) result = 37 * result + freeAspect.hashCode();

		return (int)result;
	}

	/**
	 * Überprüft auf Gleicheit. Es werden alle Membervariablen außer der Simulationsvariante bei der Überprüfung berücksichtigt. {@link #freeAspect} wird jedoch
	 * nur verglichen, falls {@link #aspect} null ist.
	 *
	 * @param obj Vergleichsobjekt.
	 *
	 * @return Ergebnis des Vergleichs.
	 *
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
    public boolean equals(Object obj) {
		boolean result = false;
		if(obj instanceof Qualifier) {
			Qualifier q = (Qualifier)obj;
			result = Objects.equals(object, q.object) && Objects.equals(attributeGroup, q.attributeGroup) && Objects.equals(aspect, q.aspect)
			         && Objects.equals(freeAspect, q.freeAspect) && simVar == q.simVar;
		}
		return result;
	}


	/**
	 * Überprüft die Qualifizierung auf Gültigkeit. Der Aspekt kann frei sein.
	 *
	 * @throws ScopeException Falls die Qualifizierung ungültig ist,
	 */
	public void valildate() throws ScopeException {
		MemberCheck.isAttributeGroupMember(object, attributeGroup);
		if(aspect != null) {
			MemberCheck.isAspectMember(attributeGroup, aspect);
		}
		else if(freeAspect == null) throw new ScopeException(InterpreterErrorMessage.ASPECT_EXPECTED);
	}

	/**
	 * Erzeugt die Beschreibung des Qualifiers.
	 *
	 * @return Beschreibung des Qualifiers. Kann ungültig sein, falls ein freier Aspekt verwendet wurde.
	 */
	public DataDescription getDataDescription() {
		return new DataDescription(attributeGroup, aspect, simVar);
	}
}
